#version 330 core

in vec3 Position_worldspace;
in vec3 Position_viewspace;
in vec3 Normal;
in vec2 Texcoord;
in mat3 TBN;

in vec4 Position_sunspace;

uniform mat4 view;
uniform mat4 model;

uniform sampler2D texture_diffuse;
uniform sampler2D texture_normal;
uniform sampler2DShadow texture_shadow;

uniform bool has_normal_texture = true;
uniform vec3 directional_light_inverse_direction = normalize(vec3(0, 2, 1));

out vec4 out_color;

const float gamma = 2.2;
const vec3 directional_light_color = vec3(0.7, 0.5, 0.3);
const vec3 point_light_position = vec3(-1.0, 1.5, 4.0);
const vec3 point_light_color = vec3(0.8, 1.0, 0.7);

void main()
{
	vec3 camera_position = vec3(inverse(view) * vec4(0, 0, 0, 1));
	vec3 frag_to_camera_direction = normalize(camera_position - Position_worldspace);

	vec3 object_color = texture(texture_diffuse, Texcoord).rgb;

	vec3 normal_map = Normal;
	if (has_normal_texture) {
		normal_map = texture(texture_normal, Texcoord).rgb;
		normal_map = normal_map * 2.0 - 1.0;
		normal_map = normalize(TBN * normal_map);
	}

	vec3 Ambient = vec3(0.05, 0.05, 0.1);

	/* Directional light */
	vec3 directional_sum;
	{
	float diffuse = max(dot(normal_map, directional_light_inverse_direction), 0.0);

	vec3 half_vector = normalize(frag_to_camera_direction + directional_light_inverse_direction);
	float specular = dot(normal_map, half_vector);
	specular = 0.5 * max(pow(specular, 256), 0.0);
	directional_sum = directional_light_color * (specular + diffuse);
	}

	/* Point light */
	vec3 point_sum;
	{
	vec3 frag_to_point_light_direction = normalize(point_light_position - Position_worldspace);
	float distance = length(point_light_position - Position_worldspace);
	float attenuation = 1.0 / (distance * distance);

	float diffuse = max(dot(normal_map, frag_to_point_light_direction), 0.0);

	vec3 half_vector = normalize(frag_to_camera_direction + frag_to_point_light_direction);
	float specular = dot(normal_map, half_vector);
	specular = 0.5 * max(pow(specular, 256), 0.0);
	point_sum = attenuation * point_light_color * (specular + diffuse);
	}

	float shadow_depth = textureProj(texture_shadow, Position_sunspace);

	vec3 final_color = (point_sum + directional_sum) * shadow_depth + Ambient;
	final_color *= object_color;
	final_color = pow(final_color, vec3(1.0 / gamma));
	out_color = vec4(final_color, 1.0);
}
